"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const fast_glob_1 = __importDefault(require("fast-glob"));
/**
 * `console.log` messages if in development environment
 * @param params - Messages to log
 */
const log = (...params) => process.env.NODE_ENV === "development" && console.log(...params);
/**
 * Eject dependencies
 */
exports.eject = async (settings = {}) => {
    const nodeModulesDir = settings.sourceDir || path_1.join(".", "node_modules");
    const destDir = settings.destDir || path_1.join(".", "ejected");
    const codeFiles = settings.codeFiles || [
        "src/**/*.{js,jsx,ts,tsx,mjs,es,es6}",
        "*.{js,jsx,ts,tsx,mjs,es,es6}"
    ];
    const updatedFiles = new Set();
    let updatedDependencies = new Set();
    await fs_extra_1.ensureDir(destDir);
    if (!(await fs_extra_1.pathExists(nodeModulesDir)))
        throw new Error("node_modules not found");
    const pkg = await fs_extra_1.readJson(path_1.join(".", "package.json"));
    let dependencies = Object.keys(pkg.dependencies);
    const files = (await fast_glob_1.default(codeFiles)).filter(file => settings.updateTestFiles
        ? true
        : !(file.includes(".test.") || file.includes(".spec.")));
    for await (const file of files) {
        let contents = await fs_extra_1.readFile(path_1.join(".", file), "utf8");
        for (const dependency of dependencies)
            if (contents.includes(dependency))
                updatedDependencies.add(dependency);
    }
    if (settings.dependenciesFilter)
        updatedDependencies = settings.dependenciesFilter(updatedDependencies);
    for await (const dependency of updatedDependencies) {
        await fs_extra_1.copy(path_1.join(nodeModulesDir, dependency), path_1.join(destDir, dependency));
        log("Copied dependency", dependency);
    }
    for await (const file of files) {
        let contents = await fs_extra_1.readFile(path_1.join(".", file), "utf8");
        const pathToSource = (path_1.join(".", file).match(/\//g) || []).length;
        for (const dependency of updatedDependencies) {
            if (contents.includes(dependency))
                updatedFiles.add(file);
            contents = contents.replace(`"${dependency}"`, `"${pathToSource > 0 ? "../".repeat(pathToSource) : "./"}${destDir}/${dependency}"`);
        }
        if (!updatedFiles.has(file))
            continue;
        await fs_extra_1.writeFile(path_1.join(".", file), contents);
        log("Updated file", file);
    }
    return { updatedFiles, updatedDependencies };
};
//# sourceMappingURL=index.js.map